package com.mortals.xhx.module.type.service.impl;
import com.mortals.framework.model.PageInfo;
import org.springframework.beans.BeanUtils;

import java.util.Map;
import java.util.function.Function;
import org.springframework.stereotype.Service;
import com.mortals.framework.service.impl.AbstractCRUDServiceImpl;
import com.mortals.framework.exception.AppException;
import com.mortals.framework.model.Context;
import com.mortals.xhx.module.type.dao.TypeDao;
import com.mortals.xhx.module.type.model.TypeEntity;
import com.mortals.xhx.module.type.service.TypeService;
import org.springframework.beans.factory.annotation.Autowired;
import com.mortals.xhx.module.type.model.TypeDetailsEntity;
import com.mortals.xhx.module.type.model.TypeDetailsQuery;
import com.mortals.xhx.module.type.service.TypeDetailsService;
import org.springframework.util.ObjectUtils;
import java.util.Date;
import java.util.Arrays;
import java.util.List;
import java.util.stream.Collectors;

import lombok.extern.slf4j.Slf4j;
/**
* TypeService
* 类型 service实现
*
* @author zxfei
* @date 2024-11-25
*/
@Service("typeService")
@Slf4j
public class TypeServiceImpl extends AbstractCRUDServiceImpl<TypeDao, TypeEntity, Long> implements TypeService {
    @Autowired
    private TypeDetailsService typeDetailsService;

    @Override
    protected void findAfter(TypeEntity params, PageInfo pageInfo, Context context, List<TypeEntity> list) throws AppException {
        fillSubData(list);
        super.findAfter(params,pageInfo, context, list);
    }
    @Override
    protected void findAfter(TypeEntity params, Context context, List<TypeEntity> list) throws AppException {
        fillSubData(list);
        super.findAfter(params, context, list);
    }

    private void fillSubData(List<TypeEntity> list) {
        List<Long> idList = list.stream().map(i -> i.getId()).collect(Collectors.toList());
        if(ObjectUtils.isEmpty(idList))return;
        Map<Long, List<TypeDetailsEntity>> typeDetailsListMap = typeDetailsService
                .find(new TypeDetailsQuery().typeIdList(idList)).parallelStream()
                .collect(Collectors.groupingBy(TypeDetailsEntity::getTypeId));
        list.forEach(item -> item.setTypeDetailsList(typeDetailsListMap.get(item.getId())));
    }

    
    @Override
    protected void saveAfter(TypeEntity entity, Context context) throws AppException {
        if(!ObjectUtils.isEmpty(entity.getTypeDetailsList())){
            entity.getTypeDetailsList().stream().peek(item->{
                item.setTypeId(entity.getId());
                item.setCreateUserId(this.getContextUserId(context));
                item.setCreateTime(new Date());
            }).count();
            typeDetailsService.save(entity.getTypeDetailsList());
        }
        super.saveAfter(entity, context);
    }

    @Override
    protected void updateAfter(TypeEntity entity, Context context) throws AppException {
        if(!ObjectUtils.isEmpty(entity.getTypeDetailsList())){
            Long[] typeDetailsIds = typeDetailsService.find(new TypeDetailsQuery().typeId(entity.getId())).stream().map(TypeDetailsEntity::getId).toArray(Long[]::new);
            typeDetailsService.remove(typeDetailsIds,context);
            entity.getTypeDetailsList().stream().peek(item ->{
                item.setTypeId(entity.getId());
                item.setCreateUserId(this.getContextUserId(context));
                item.setCreateTime(new Date());
                item.setUpdateUserId(this.getContextUserId(context));
                item.setUpdateTime(new Date());
            }).count();
            typeDetailsService.save(entity.getTypeDetailsList());
        }
        super.updateAfter(entity, context);
    }

    @Override
    protected void removeAfter(Long[] ids, Context context, int result) throws AppException {
        List<TypeDetailsEntity> typeDetailslist = typeDetailsService.find(new TypeDetailsQuery().typeIdList(Arrays.asList(ids)));
        typeDetailsService.removeList(typeDetailslist,context);
        super.removeAfter(ids, context, result);
    }
}